import hashlib
import logging
from pathlib import PurePath
from shutil import copyfile
from urllib.parse import urlparse

import requests

logger: logging.Logger = logging.getLogger(__name__)


def build_path(content: bytes, source_path: str, dest: str) -> str:
    hashsha = hashlib.sha256(content)
    path = PurePath(source_path)
    suffix = path.suffix
    return str(path.joinpath(dest, hashsha.hexdigest() + suffix))


async def copy_local_file(source_path: str, destdir: str) -> str:
    with open(source_path, "rb") as fopen:
        content = fopen.read()
    dest_path = build_path(content, source_path, destdir)
    copyfile(source_path, dest_path)
    return dest_path


async def download_file(source: str, source_path: str, dest: str) -> str:
    """Download the file from http(s)"""
    resp = requests.get(source, timeout=60)
    resp.raise_for_status()
    dest_path = build_path(resp.content, source_path, dest)
    with open(dest_path, "wb") as fopen:
        fopen.write(resp.content)
    return dest_path


async def download(source: str, dest_dir: str) -> str:
    """
    Determine the protocol to fetch the document:
    file://,
    http://,
    s3:// ...
    """
    parsedurl = urlparse(source)
    logger.info("download %s, %s", parsedurl.scheme, parsedurl.path)
    if parsedurl.scheme in ["file", ""]:
        return await copy_local_file(parsedurl.path, dest_dir)
    if parsedurl.scheme in ["http", "https"]:
        return await download_file(source, parsedurl.path, dest_dir)

    raise AttributeError(
        f"Unsupported file source: scheme={parsedurl.scheme} - path={parsedurl.path}"
    )



from abc import ABC, abstractmethod


class BaseOcr(ABC):
    name: str = "baseocr"

    @abstractmethod
    def get_pages(self, filepath: str) -> list[str]:
        raise NotImplementedError("")

    @abstractmethod
    def ocr(self, document_path: str) -> list[str]:
        raise NotImplementedError("")



from typing_extensions import TypeAlias

from .ocr.googlevision import GoogleVisionOcr
from .ocr.tesseract import TesseractOcr

OCR: TypeAlias = TesseractOcr | GoogleVisionOcr


class OpticR:
    processors: dict[str, type[OCR]] = {
        "tesseract": TesseractOcr,
        "google-vision": GoogleVisionOcr,
    }

    def __init__(self, processor: str = "tesseract") -> None:
        self.processor: OCR = self.processors[processor]()

    def get_pages(self, filepath: str) -> list[str]:
        return self.processor.get_pages(filepath)

    def processor_name(self) -> str:
        return self.processor.name



from pathlib import PurePath

import pdf2image
import pytesseract
from PIL import Image

from .baseocr import BaseOcr


class TesseractOcr(BaseOcr):
    name: str = "tesseract"

    def __init__(self) -> None:
        pass

    def get_pages(self, filepath: str) -> list[str]:
        return self.ocr(filepath)

    def ocr(self, document_path: str) -> list[str]:
        path = PurePath(document_path)
        if path.suffix == ".pdf":
            pages = pdf2image.convert_from_path(str(path))
        else:
            pages = [Image.open(str(path))]
        pages_ocr = []
        for page in pages:
            pages_ocr.append(pytesseract.image_to_string(page))
        return pages_ocr



from .baseocr import BaseOcr


class GoogleVisionOcr(BaseOcr):
    name: str = "google-vision"

    def get_pages(self, filepath: str) -> list[str]:
        raise NotImplementedError

    def ocr(self, document_path: str) -> list[str]:
        raise NotImplementedError



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['opticr', 'opticr.ocr']

package_data = \
{'': ['*']}

install_requires = \
['ocrmypdf>=14.0.0,<15.0.0',
 'pdf2image>=1.16.0,<2.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'pytesseract>=0.3.10,<0.4.0',
 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'opticr',
    'version': '0.2.0',
    'description': 'expose a single interface and API to few OCR tools',
    'long_description': '# opticr\n\nPython library to expose a single interface and API to few OCR tools (google vision, Tesseract)\n\n## Install\n### Required binaries available in the $PATH\n#### poppler-utils (pdf2image)\n\n[https://github.com/Belval/pdf2image#how-to-install](https://github.com/Belval/pdf2image#how-to-install)\n\n#### tesseract\n\n[https://tesseract-ocr.github.io](https://tesseract-ocr.github.io/tessdoc/Home.html)\n\n### Install OpticR\n#### With pip\n\n``` shell\npip install opticr\n```\n\n#### With poetry\n\n``` shell\npoetry add opticr\n```\n\nor to get the latest \'dangerous\' version\n\n```\npoetry add  git+https://github.com/lzayep/opticr@main\n```\n\n## Usage\n\n``` python\nfrom opticr import OpticR\n\nocr = OpticR("tesseract")\npathtofile = "test/contract.pdf\npages: list[str] = ocr.get_pages(pathtofile)\n\n```\n\nWith google-vision:\n\n``` python\nfrom opticr import OpticR\n\nocr = OpticR("google-vision", options={"google-vision": {"auth": {"token": ""}}})\n\n# file could come from an URL\npathtofile = "https://example.com/contract.pdf\npages: list[str] = ocr.get_pages(pathtofile)\n\n```\n\nCache the result, if the file as already been OCR return immediatly the previous result.\nResult are stored temporarly in the local storage or shared storage such as Redis.\n``` python\nfrom opticr import OpticR\n\nocr = OpticR("tesseract", options={"cache":\n                         {"backend": "redis", redis: "redis://"}}\n\n# file could come from an URL\npathtofile = "https://example.com/contract.pdf\npages: list[str] = ocr.get_pages(pathtofile, cache=True)\n\n```\n',
    'author': 'lzayep',
    'author_email': 'ec@lza.sh',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
